@@ -1,4 +1,7 @@
-{-# LANGUAGE CPP, GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE CPP #-}
+#ifdef LANGUAGE_DeriveDataTypeable
+{-# LANGUAGE DeriveDataTypeable #-}
+#endif
 ----------------------------------------------------------------------------
 -- |
 -- Module     : Data.Proxy
@@ -7,7 +10,7 @@
 --
 -- Maintainer  : Edward Kmett <ekmett@gmail.com>
 -- Stability   : experimental
--- Portability : generalized newtype deriving
+-- Portability : portable
 --
 -------------------------------------------------------------------------------
 
@@ -25,11 +28,12 @@ module Data.Proxy
 import Control.Applicative (Applicative(..))
 import Data.Traversable (Traversable(..))
 import Data.Foldable (Foldable(..))
-import Data.Default (Default(..))
+#ifdef LANGUAGE_DeriveDataTypeable
 import Data.Data (Data,Typeable)
+#endif
 import Data.Ix (Ix(..))
 import Data.Tagged
-import Data.Semigroup
+import Data.Monoid
 #ifdef __GLASGOW_HASKELL__
 import GHC.Arr (unsafeIndex, unsafeRangeSize)
 #endif
@@ -37,7 +41,7 @@ import GHC.Arr (unsafeIndex, unsafeRangeSize)
 data Proxy p = Proxy deriving 
   ( Eq, Ord, Show, Read
 #ifdef LANGUAGE_DeriveDataTypeable
-  , Data,Typeable
+  , Data, Typeable
 #endif
   )
 
@@ -52,9 +56,6 @@ instance Enum (Proxy s) where
     enumFromThenTo _ _ _ = [Proxy]
     enumFromTo _ _ = [Proxy]
 
-instance Semigroup (Proxy s) where
-    _ <> _ = Proxy
-
 {- 
 Work around for the following GHC bug with deriving Ix instances with a phantom type:
 
@@ -130,10 +131,6 @@ instance Traversable Proxy where
     sequence _ = return Proxy
     {-# INLINE sequence #-}
 
-instance Default (Proxy s) where
-    def = Proxy
-    {-# INLINE def #-}
-
 -- | Some times you need to change the proxy you have lying around.
 -- Idiomatic usage is to make a new combinator for the relationship 
 -- between the proxies that you want to enforce, and define that 
